"""Bitcoin Client API module."""
from typing import List
from blockpulsar.api import BlockPulsarAPI
from blockpulsar.response_obj import ResponseObj


class BTCClient:
    """
    BTCClient class provides Blockpulsar bitcoin API methods.
    """
    def __init__(self, api_key: str, secret_key: str):
        self.api = BlockPulsarAPI(api_key, secret_key)

    def get_best_block_hash(self):
        rv = self.api.get('btc/blockchain/getbestblockhash')
        if 'data' in rv:
            return rv['data']
        raise Exception(rv['error'])

    def get_block(self, blockhash: str):
        res = self.api.get('btc/blockchain/getblock', {'blockhash': blockhash})
        return ResponseObj(res)

    def get_blockchain_info(self):
        res = self.api.get('btc/blockchain/getblockchaininfo')
        return ResponseObj(res)

    def get_block_count(self):
        rv = self.api.get('btc/blockchain/getblockcount')
        if 'data' in rv:
            return rv['data']
        raise Exception(rv['error'])

    def get_block_hash(self, height: int):
        rv = self.api.get('btc/blockchain/getblockhash', {'height': height})
        if 'data' in rv:
            return rv['data']
        raise Exception(rv['error'])

    def get_block_header(self, blockhash: str):
        params = {'blockhash': blockhash}
        res = self.api.get('btc/blockchain/getblockheader', params)
        return ResponseObj(res)

    def get_block_stats(self, hash_or_height):
        params = {'hash_or_height': hash_or_height}
        res = self.api.get('btc/blockchain/getblockstats', params)
        return ResponseObj(res)

    def get_chain_tips(self):
        return self.api.get('btc/blockchain/getchaintips')

    def get_chain_tx_stats(self, blockhash: str, nblocks: int = 30):
        params = {'blockhash': blockhash, 'nblocks': nblocks}
        res = self.api.get('btc/transaction/getchaintxstats', params)
        return ResponseObj(res)

    def get_mempool_ancestors(self, txid: str):
        params = {'txid': txid}
        return self.api.get('btc/transaction/getmempoolancestors', params)

    def get_mempool_descendants(self, txid: str):
        params = {'txid': txid}
        return self.api.get('btc/transaction/getmempooldescendants', params)

    def get_mempool_entry(self, txid: str):
        res = self.api.get('btc/transaction/getmempoolentry', {'txid': txid})
        return ResponseObj(res)

    def get_mempool_info(self):
        res = self.api.get('btc/transaction/getmempoolinfo')
        return ResponseObj(res)

    def get_raw_mempool(self):
        return self.api.get('btc/transaction/getrawmempool')

    def get_tx_out(self, txid: str, n: int, include_mempool: bool = True):
        params = {'txid': txid, 'n': n, 'include_mempool': include_mempool}
        res = self.api.get('btc/transaction/gettxout', params)
        return ResponseObj(res)

    def get_tx_out_proof(self, txids: List[str], blockhash: str = ''):
        params = {'txids': txids, 'blockhash': blockhash}
        return self.api.get('btc/blockchain/gettxoutproof', params)

    def get_difficulty(self):
        rv = self.api.get('btc/mining/getdifficulty')
        if 'data' in rv:
            return rv['data']
        raise Exception(rv['error'])

    def get_mining_info(self):
        res = self.api.get('btc/mining/getmininginfo')
        return ResponseObj(res)

    def get_network_hashps(self, nblocks: int = 120, height: int =- 1):
        params = {'nblocks': nblocks, 'height': height}
        rv = self.api.get('btc/mining/getnetworkhashps', params)
        if 'data' in rv:
            return rv['data']
        raise Exception(rv['error'])

    def get_raw_transaction(self, txid: str, blockhash: str = ''):
        params = {'txid': txid, 'blockhash': blockhash}
        res = self.api.get('btc/transaction/getrawtransaction', params)
        return ResponseObj(res)

    def validate_address(self, address: str):
        res = self.api.get('btc/address/validateaddress', {'address': address})
        return ResponseObj(res)
